package inf1010.pusle.adventure;

import java.util.Map;
import java.util.HashMap;
import java.util.Scanner;

/**
 * Main class for our text adventure.
 */
public class Adventure implements Runnable {

    /** Used to scan the CLI/shell. */
    private final Scanner cliScanner = new Scanner(System.in);

    /** The player. */
    private Character player = new Character("self", "You look dashing!",
					     100, 20, 20, null);

    /**
     * Create an Adventure.
     * Oh joy!
     * Initialization code goes here.
     */
    public Adventure() {
	try {
	    player.location = MapLoader.loadMap();
	    if (player.location == null) {
		System.err.println("What the rape?");
		System.exit(500);
	    }
		
	} catch (Exception e) {
	    System.err.printf("Could not load map:%n%s%n", e.getMessage());
	    System.exit(500);
	}
    }

    /**
     * Super-simple entry-point.
     *
     * @param args Command-line arguments
     */
    public static void main(String args[]) {
	new Adventure().run();
    }

    /**
     * Main thread running point.
     */
    public void run() {
	motd();
	player.location.printDescription();
	while(true) {
	    prompt();
	}
    }
	
    /**
     * Prints a prompt and parses the input.
     */
    private void prompt() {
	System.out.printf(" > ");
		
	// Super-simple command reading.
	Scanner cmdScanner = cmdScanner = new Scanner(cliScanner.nextLine().toLowerCase());
	// The user entered an empty line.
	if (!cmdScanner.hasNext()) {
	    return;
	}

	String verb = cmdScanner.next();		
	String firstObject = cmdScanner.hasNext() ? cmdScanner.next() : null;
	String secondObject = cmdScanner.hasNext() ? cmdScanner.next() : null;

	// Hard-coded verbs.
	// Bad code, but for such a simple game, this is okay.
	// LOOK
	if (verb.equals("look")) {
	    if (firstObject != null) {
		Entity fO = find(firstObject);
		if (fO == null) {
		    System.out.printf("%nYou stare into space, but don't see anything of interest!%n");
		} else {
		    fO.printDescription();
		    if (fO instanceof Character) {
			player.consider((Character) fO);
		    }
		}
	    } else {
		player.location.printDescription();
	    }

	// PRINT STATS - FOR DEBUG PURPOSES
	// This could be replaced by some printouts regarding wounds
	// after a fight.
	} else if (verb.equals("stats")) {
	    System.out.printf("HP: %d/%d%n", player.hp, player.maxHp);
	    System.out.printf("Attack: %d (%s%d)%n", player.baseAttack,
			      player.attack > player.baseAttack ? "+" : "",
			      player.attack - player.baseAttack);
	    System.out.printf("Defense: %d (%s%d)%n", player.baseDefense,
			      player.defense > player.baseDefense ? "+" : "",
			      player.defense - player.baseDefense);

	// PRINT INVENTORY
	} else if (verb.equals("inventory")) {
	    if (player.items.isEmpty()) {
		System.out.printf("Your pockets are full of lint, but nothing of interest.%n");
	    } else {
		System.out.printf("Inventory:%n");
		for (Item it : player.items) {
		    System.out.printf("- %s ", it.name);
		    if (it instanceof Equipment
			&& player.equipment.containsValue((Equipment) it)) {
			System.out.printf("(equipped)");
		    }
		    System.out.println();
		}
	    }

	// GET AN ITEM
	} else if (verb.equals("get")) {
	    Item it = player.location.items.get(new Item(firstObject, null));
	    if (it == null) {
		System.out.printf("You grab som air and put it in your pockets.%n");
	    } else {
		player.location.items.remove(it);
		player.items.add(it);
		System.out.printf("You pick up the %s.%n", it.name);
	    }

	// DROP AN ITEM
	} else if (verb.equals("drop")) {
	    Item it = player.items.get(new Item(firstObject, null));
	    if (it == null) {
		System.out.printf("You throw away some pocket lint.%n");
	    } else {
		player.drop(it);
	    }

	// EQUIP AN ITEM
	} else if (verb.equals("equip")) {
	    Item it = player.items.get(new Item(firstObject, null));
	    if (it == null) {
		System.out.printf("You equip some air and feel much stronger.%n");
	    } else if (!(it instanceof Equipment)) {
		System.out.printf("You try to equip the %s, but fail miserably.%n",
				  it.name);
	    } else {
		player.equip((Equipment) it, false);
	    }

	// UNEQUIP AN ITEM
	} else if (verb.equals("unequip")) {
	    Item it = player.items.get(new Item(firstObject, null));
	    if (it == null) {
		System.out.printf("You take off the air and feel weak.%n");
	    } else if (!(it instanceof Equipment)) {
		System.out.printf("You keep trying to remove the %s, but suddenly realize it's not equipped.%n",
				  it.name);
	    } else {
		player.unequip((Equipment) it, false);
	    }

	// QUIT THE GAME
	} else if (verb.equals("quit")) {
	    System.out.printf("Good bye!%n");
	    System.exit(0);

	// TRY TO KILL A CHARACTER
	} else if (verb.equals("kill") || verb.equals("fight")) {
	    if (firstObject == null) {
		System.out.printf("You feel like killing someone, but whom should you kill?%n");
		return;
	    }
			
	    Character opponent = player.location.npcs.get(new Character(firstObject, null, 0, 0, 0, null));
	    if (opponent == null) {
		System.out.printf("You swing at the air a bit, but then realize they're not here.%n");
		return;
	    }
			
	    if (player.kill(opponent)) {
		return;
	    }
			
	    System.exit(0);

	// SECRET CHEAT
	} else if (verb.equals("win")) {
	    System.out.printf("You winz teh 64|\\/|3!%nGood bye!%n");
	    System.exit(0);

	// PRINT COMMANDS
	} else if (verb.equals("help")) {
	    System.out.println("Available commands:");
	    System.out.println("- look");
	    System.out.println("- inventory");
	    System.out.println("- get");
	    System.out.println("- drop");
	    System.out.println("- equip");
	    System.out.println("- unequip");
	    System.out.println("- kill");
	    System.out.println("- north");
	    System.out.println("- east");
	    System.out.println("- south");
	    System.out.println("- west");
	    System.out.println("- quit");

	// CHECK IF THE COMMAND IS A DIRECTION
	} else {
	    Direction d = Direction.getDirection(verb);
	    if(d != null) {
		Room tmp = player.location.getRoom(d);
		if (tmp != null) {
		    System.out.printf("You go %s.%n%n", d.text);
		    player.location = tmp;
		    player.location.printDescription();
		} else {
		    System.out.printf("You can't go that way.%n%n");
		}
	    } else
		System.out.printf("%s %s.%n",
				  "You want to do something,",
				  "but you don't understand what it is");
	}
    }
	
    /**
     * Tries to find an entity in the room with the given name.
     *
     * @param name The name of the entity that's being looked for.
     * @return An entity with a name similar to the given name.
     */
    private Entity find(String name) {
	Item tmp = new Item(name, null);
	Character tmp2 = new Character(name, null, 0, 0, 0, null);
	Entity ret = player.items.get(tmp);
		
	if (ret == null)
	    ret = player.location.items.get(tmp);
		
	if (ret == null) {
	    ret = player.location.npcs.get(tmp2);
	}

	if (ret == null) {
	    ret = player.compareTo(tmp2) == 0 ? player : null;
	}

	return ret;
    }

    /**
     * Message of the day.
     */
    private void motd() {
	System.out.printf("%n%nWelcome to the most awesomest game ever!%n%n");
	System.out.printf("For a list of commands, type help%n");
    }
}
