package inf1010.pusle.adventure;

/**
 * An enum representing directions in the game.
 * Using an enum like this simplifies things considerably.
 */
enum Direction {

	NORTH("north", "n"),
	EAST("east", "e"),
	SOUTH("south", "s"),
	WEST("west", "w");
	
	/**
	 * Text and alternate text associated with the Direction.
	 */
	String text, alt;
	
	/**
	 * Creates a new Direction with an associated text and alternate text.
	 *
	 * @param text The text associated with this Direction.
	 * @param alt An alternate/abbreviated text.
	 */
	private Direction(String text, String alt) {
		this.text = text;
		this.alt = alt;
	}

	/**
	 * Gets the Direction matching the given string.
	 *
	 * @param text The text to match to a direction.
	 * @return The Direction matching the text or null if there's no match.
	 */
	public static Direction getDirection(String text) {
		for (Direction d : values()) {
		    if (d.text.equals(text) || d.alt.equals(text)) {
				return d;
		    }
		}

		return null;
	}
}